using System;
using System.Collections;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Lifetime;
using System.Runtime.Remoting.Messaging;

using Belikov.GenuineChannels;
using Belikov.GenuineChannels.Connection;
using Belikov.GenuineChannels.DirectExchange;
using Belikov.GenuineChannels.DotNetRemotingLayer;
using Belikov.GenuineChannels.GenuineTcp;
using Belikov.GenuineChannels.Logbook;
using Belikov.GenuineChannels.TransportContext;

namespace Server
{
	/// <summary>
	/// Chat server implements server that configures Genuine Server TCP Channel and implements
	/// chat server behavior.
	/// </summary>
	class ChatServer
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			try
			{
				// setup .NET remoting
				System.Configuration.ConfigurationSettings.GetConfig("DNS");
				GenuineGlobalEventProvider.GenuineChannelsGlobalEvent += new GenuineChannelsGlobalEventHandler(GenuineChannelsEventHandler);

				Hashtable properties = new Hashtable();
				properties["port"] = "8737";
				properties["MaxTimeSpanToReconnect"] = "5000";
				GenuineTcpChannel genuineTcpChannel = new GenuineTcpChannel(properties, null, null);
				ChatServer.TransportContext = genuineTcpChannel.ITransportContext;
				ChannelServices.RegisterChannel(genuineTcpChannel);

				ChatServer.TransportContext.DirectExchangeManager.RegisterServerService("/Test/DataReceiver", new DirectDataReceiver());
				RemotingServices.Marshal(new DataReceiver(), "DataReceiver.rem");

				Console.WriteLine("Server has been started. Press enter to exit.");
				Console.ReadLine();
			}
			catch(Exception ex)
			{
				Console.WriteLine("Exception: {0}. Stack trace: {1}.", ex.Message, ex.StackTrace);
			}
		}

		public static ITransportContext TransportContext;

		/// <summary>
		/// Handles network events.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">Event parameters.</param>
		public static void GenuineChannelsEventHandler(object sender, GenuineEventArgs e)
		{
			Console.WriteLine("Global event: {0}\r\nUrl: {1}", e.EventType, e.HostInformation  == null ? "<unknown>" : e.HostInformation.ToString());

			if (e.EventType == GenuineEventType.GTcpConnectionAccepted)
			{
				ConnectionAcceptedCancellableEventParameter connectionAcceptedCancellableEventParameter = (ConnectionAcceptedCancellableEventParameter) e.AdditionalInfo;
				Console.WriteLine("An incoming connection from {0} is being accepted.", connectionAcceptedCancellableEventParameter.IPEndPoint.ToString());
			}
		}

	}
}
